/*
 * Copyright (c) 2005-2010 KOM  Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.sensor.generic;

import de.tudarmstadt.maki.simonstrator.api.component.overlay.OverlayContact;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.generic.operation.ProcessingLog;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;

public class SingleValueDataRepresentation implements DataRepresentation {
	private final int _dataID;

	private long[] _vehicleIDs;

	private OverlayContact _requestor;

	private double _data;

	private final Location _position;
	private final long _timestamp;

	private ProcessingLog _processingLog;

	private SensorConfiguration _sensorConfiguration;

	public SingleValueDataRepresentation(int pDataID, double pMeasure, Location pPosition, long pTimestamp,
			long... pVehicleIDs) {
		this(pDataID, pMeasure, pPosition, pTimestamp, new ProcessingLog(pDataID), pVehicleIDs);
	}

	public SingleValueDataRepresentation(int pDataID, double pMeasure, Location pPosition, long pTimestamp,
			ProcessingLog pProcessingLog, long... pVehicleIDs) {
		_data = pMeasure;

		_dataID = pDataID;

		_vehicleIDs = pVehicleIDs;

		_position = pPosition;
		_timestamp = pTimestamp;

		_processingLog = pProcessingLog;
	}

	public SingleValueDataRepresentation(int pDataID, double pMeasure, Location pPosition, long pTimestamp,
			SensorConfiguration pSensorConfiguration, long... pVehicleIDs) {
		this(pDataID, pMeasure, pPosition, pTimestamp, new ProcessingLog(pDataID), pVehicleIDs);

		_sensorConfiguration = pSensorConfiguration;
	}

	@Override
	public int getDataID() {
		return _dataID;
	}

	public ProcessingLog getProcessingLog() {
		return _processingLog;
	}

	@Override
	public void setRequestor(OverlayContact pRequestor) {
		_requestor = pRequestor;
	}

	@Override
	public OverlayContact getRequestor() {
		return _requestor;
	}

	@Override
	public Double getData() {
		return _data;
	}

	@Override
	public int getTransmissionSize() {
		return Double.BYTES + Long.BYTES + _position.getTransmissionSize()
				+ Integer.BYTES * _vehicleIDs.length;
	}

	@Override
	public Location getLocation() {
		return _position;
	}

	@Override
	public long getTimestamp() {
		return _timestamp;
	}

	@Override
	public long[] getVehicleIDs() {
		return _vehicleIDs;
	}

	public SensorConfiguration getSensorConfiguration() {
		return _sensorConfiguration;
	}

	@Override
	public double getExpectationValue() {
		return _data;
	}

	public void setValue(double pRealValue) {
		if (pRealValue < 0) {
			System.out.println();
		}
		_data = pRealValue;
	}
}
