package de.tudarmstadt.maki.simonstrator.api.component.sensor.generic;

import java.util.Arrays;

import de.tudarmstadt.maki.simonstrator.api.component.overlay.OverlayContact;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.generic.operation.ProcessingLog;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;

public class VectorBasedDataRepresentation implements DataRepresentation {
    private final int _dataID;

	private long[] _vehicleIDs;

	private OverlayContact _requestor;

    private final double[] _data;

    private final Location _position;
    private final long _timestamp;

	private ProcessingLog _processingLog;

	private SensorConfiguration _sensorConfiguration;

	public VectorBasedDataRepresentation(int pDataID, int pAmountOfStates, Location pPosition, long pTimestamp,
			long... pVehicleIDs) {
		this(pDataID, pAmountOfStates, pPosition, pTimestamp, null, pVehicleIDs);
	}

	public VectorBasedDataRepresentation(int pDataID, int pAmountOfStates, Location pPosition, long pTimestamp,
			SensorConfiguration pSensorConfiguration, long... pVehicleIDs) {
		this(pDataID, new double[pAmountOfStates], pPosition, pTimestamp, pSensorConfiguration, pVehicleIDs);
		Arrays.fill(_data, 1 / (double) pAmountOfStates);
	}

	public VectorBasedDataRepresentation(int pDataID, double[] pMeasure, Location pPosition, long pTimestamp,
			long... pVehicleIDs) {
		this(pDataID, pMeasure, pPosition, pTimestamp, new ProcessingLog(pDataID), pVehicleIDs);
	}

	public VectorBasedDataRepresentation(int pDataID, double[] pMeasure, Location pPosition, long pTimestamp,
			ProcessingLog pProcessingLog, long... pVehicleIDs) {
		_data = pMeasure;

		_dataID = pDataID;

		_vehicleIDs = pVehicleIDs;

		_position = pPosition;
		_timestamp = pTimestamp;

		_processingLog = pProcessingLog;
	}

	public VectorBasedDataRepresentation(int pDataID, double[] pMeasure, Location pPosition, long pTimestamp,
			SensorConfiguration pSensorConfiguration, long... pVehicleIDs) {
		this(pDataID, pMeasure, pPosition, pTimestamp, new ProcessingLog(pDataID), pVehicleIDs);

		_sensorConfiguration = pSensorConfiguration;
	}

	@Override
	public int getDataID() {
		return _dataID;
	}

	public ProcessingLog getProcessingLog() {
		return _processingLog;
	}

	@Override
	public void setRequestor(OverlayContact pRequestor) {
		_requestor = pRequestor;
	}

	@Override
	public OverlayContact getRequestor() {
		return _requestor;
	}

	@Override
	public double[] getData() {
        return _data;
    }

    @Override
    public int getTransmissionSize() {
		return _data.length * Double.BYTES + Long.BYTES + _position.getTransmissionSize()
				+ Integer.BYTES * _vehicleIDs.length;
    }

	@Override
	public Location getLocation() {
		return _position;
	}

	@Override
	public long getTimestamp() {
		return _timestamp;
	}

	@Override
	public long[] getVehicleIDs() {
		return _vehicleIDs;
	}

	public SensorConfiguration getSensorConfiguration() {
		return _sensorConfiguration;
	}

	public void normalize() {
		double sum = 0;

		for (int i = 0; i < _data.length; i++) {
			sum += _data[i];
		}

		for (int i = 0; i < _data.length; i++) {
			_data[i] /= sum;
		}
	}

	@Override
	public double getExpectationValue() {
		double[] data = getData();

		double sum = 0;

		for (int i = 0; i < data.length; i++) {
			sum += i * data[i];
		}

		return sum;
	}
}
