/*
 * Copyright (c) 2005-2010 KOM  Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.sensor.generic.operation;

public abstract class BinaryOperationProvider implements ProcessingOperationProvider {

	@Override
	public double performOperation(double pValue1, double pValue2) {
		if (pValue1 == (int) pValue1 && pValue2 == (int) pValue2) {
			return performBinaryOperation((int) pValue1, (int) pValue2);
		}
		throw new AssertionError(
				getClass().getSimpleName() + " requires integral values, got " + pValue1 + " and " + pValue2);
	}

	@Override
	public double[] performOperation(double[] pData1, double[] pData2) {
		int max = 0;
		for (int i = 0; i < pData1.length; i++) {
			for (int j = 0; j < pData2.length; j++) {
				int binaryResult = performBinaryOperation(i, j);
				if (binaryResult > max) {
					max = binaryResult;
				}
			}
		}

		double[] result = new double[max + 1];

		for (int i = 0; i < pData1.length; i++) {
			for (int j = 0; j < pData2.length; j++) {
				double prob1 = pData1[i];
				double prob2 = pData2[j];

				double totalProbability = prob1 * prob2;

				if (totalProbability != 0) {
					int value = performBinaryOperation(i, j);

					result[value] += totalProbability;
				}
			}
		}

		return result;
	}

	@Override
	public int getMaxValue(int pFirstMaxValue, int pSecondMaxValue) {
		return (int) Math.ceil(performOperation(pFirstMaxValue, pSecondMaxValue));
	}

	protected abstract int performBinaryOperation(int pValue1, int pValue2);
}
