package de.tudarmstadt.maki.simonstrator.api.component.service.in_network_processing;

import de.tudarmstadt.maki.simonstrator.api.common.Transmitable;
import de.tudarmstadt.maki.simonstrator.api.component.overlay.OverlayContact;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;

public class ServiceRequest implements Transmitable {
    private int _serviceID;

    private OverlayContact _requestor;

    private AccuracyCalculation _accuracyCalculation;
    private double _requiredAccuracy;

    private Location _location = null;
    private long _timestamp = -1;

	private boolean _isLocal = false;

	private boolean _noLocal = false;

    public ServiceRequest(OverlayContact pRequestor, int pServiceID) {
        _serviceID = pServiceID;
        _requestor = pRequestor;
    }

    public ServiceRequest(OverlayContact pRequestor, int pServiceID, Location pLocation) {
        this(pRequestor, pServiceID);
        _location = pLocation;
    }

    public ServiceRequest(OverlayContact pRequestor, int pServiceID, Location pLocation, long pTimestamp) {
        this(pRequestor, pServiceID, pLocation);
        _timestamp = pTimestamp;
    }

    public ServiceRequest(OverlayContact pRequestor, int pServiceID, Location pLocation,
            long pTimestamp, AccuracyCalculation pAccuracy, double pRequiredAccuracy) {
        this(pRequestor, pServiceID, pLocation, pTimestamp);
        _accuracyCalculation = pAccuracy;
        _requiredAccuracy = pRequiredAccuracy;
    }

    public ServiceRequest(OverlayContact pRequestor, int pServiceID, Location pLocation, long pTimestamp,
            AccuracyCalculation pAccuracy, double pRequiredAccuracy, boolean pLocal) {
        this(pRequestor, pServiceID, pLocation, pTimestamp, pAccuracy, pRequiredAccuracy);
		_isLocal = pLocal;
    }

    public int getServiceID() {
        return _serviceID;
    }

    public AccuracyCalculation getAccuracyCalculation() {
        return _accuracyCalculation;
    }

    public double getRequiredAccuracy() {
        return _requiredAccuracy;
    }

    public OverlayContact getRequestor() {
        return _requestor;
    }

    public Location getLocation() {
        return _location;
    }

    public long getTimestamp() {
        return _timestamp;
    }

    public boolean isLocal() {
		return _isLocal;
    }

    @Override
    public int getTransmissionSize() {
        int size = Integer.BYTES;

        if (_accuracyCalculation != null) {
            size += _accuracyCalculation.getTransmissionSize();
        }
        if (_location != null) {
            size += _location.getTransmissionSize();
        }
        if (_timestamp != -1) {
            size += Long.BYTES;
        }
        size += _requestor.getTransmissionSize();

        return size;
    }

	public void setNoLocal(boolean pNoLocal) {
		_noLocal = pNoLocal;
	}

	public boolean isNoLocal() {
		return _noLocal;
	}

    public void setLocal(boolean pLocal) {
		_isLocal = pLocal;
    }

}
