package de.tudarmstadt.maki.simonstrator.api.component.vehicular.data;

import java.util.HashMap;
import java.util.Map;

import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.data.map.ComplexSensorValueMap;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.data.map.IntegralSensorValueMap;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.data.map.SensorValueMap;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.data.map.SimpleSensorValueMap;

public class SensorValueMapContainer {
	private static final SensorValueMapContainer _map = new SensorValueMapContainer();

	private Map<Integer, SensorValueMap> _sensorValues = new HashMap<>();

	private SensorValueMapContainer() {
	}

	public static SensorValueMapContainer getMap() {
		return _map;
	}

	public void registerSensorSimpleVariable(int pSensorID, double pMean, double pMax, double pTemporalChange,
			double pGeographicalChange) {
		_sensorValues.put(pSensorID, new SimpleSensorValueMap(pMean, pMax, pTemporalChange, pGeographicalChange));
	}

	public void registerSensorComplexVariable(int pSensorID, double pMean, double pMax, double pTemporalChange,
			double pGeographicalChange, double pWorldWidth, double pWorldHeight) {
		_sensorValues.put(pSensorID, new ComplexSensorValueMap(pMean, pMax, pTemporalChange, pGeographicalChange,
				pWorldWidth, pWorldHeight));
	}

	public void registerSensorIntegralVariable(int pSensorID, double pMean, double pMax, double pTemporalChange,
			double pGeographicalChange, double pWorldWidth, double pWorldHeight) {
		_sensorValues.put(pSensorID, new IntegralSensorValueMap(pMean, pMax, pTemporalChange, pGeographicalChange,
				pWorldWidth, pWorldHeight));
	}

	public double getMaxValue(int pSensorID) {
		SensorValueMap sensorValueMap = _sensorValues.get(pSensorID);
		return sensorValueMap.getMaxValue();
	}

	public double getRealValue(int pSensorID, Location pLocation) {
		SensorValueMap sensorValueMap = _sensorValues.get(pSensorID);
		return sensorValueMap.getValueAt(pLocation);
	}

	public double getRealValue(int pSensorID, Location pLocation, long pTimestamp) {
		SensorValueMap sensorValueMap = _sensorValues.get(pSensorID);
		return sensorValueMap.getValueAt(pLocation, pTimestamp);
	}
}
