package de.tudarmstadt.maki.simonstrator.api.component.vehicular.data.map;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

import de.tudarmstadt.maki.simonstrator.api.Event;
import de.tudarmstadt.maki.simonstrator.api.EventHandler;
import de.tudarmstadt.maki.simonstrator.api.Randoms;
import de.tudarmstadt.maki.simonstrator.api.Time;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;

public class SimpleSensorValueMap implements SensorValueMap, EventHandler {

    private Random _random = Randoms.getRandom(getClass());

	private final long INTERVAL = Time.MINUTE;
	private long _lastTrendSet = 0;
	private double _trend = 0;

    private double _mean;
    private double _max;

    private Map<Long, Double> _pastValues = new HashMap<>();
    private List<Long> _availableTimestamps = new ArrayList<>();

    private double _temporalChange;
    private double _geographicalChange;

    public SimpleSensorValueMap(double pMean, double pMax, double pTemporalChange, double pGeographicalChange) {
        _mean = pMean;

        updatePastValuesMap();

		setTrend();

        _max = pMax;

        _temporalChange = pTemporalChange;
        _geographicalChange = pGeographicalChange;

        Event.scheduleWithDelay(Time.SECOND, this, null, 0);
    }

	private void setTrend() {
		_trend = _random.nextDouble();
		_lastTrendSet = Time.getCurrentTime();
	}

	@Override
    public double getValueAt(Location pLocation) {
        return _mean;
    }

    @Override
    public double getValueAt(Location pLocation, long pTimestamp) {
        for (int i = _availableTimestamps.size() - 1; i >= 0; i--) {
            long timestamp = _availableTimestamps.get(i);
            if (timestamp <= pTimestamp) {
                double oldMean = _pastValues.get(timestamp);
                return oldMean;
            }
        }
        return Double.NEGATIVE_INFINITY;
    }

    @Override
    public void eventOccurred(Object pContent, int pType) {
		if (Time.getCurrentTime() - INTERVAL >= _lastTrendSet) {
			setTrend();
		}

        double oldMean = _mean;
        double nextDouble = _random.nextDouble();

		double maximumChange = _temporalChange / 2.0;
		
		double factor;
		if (nextDouble < _trend) {
			factor = -nextDouble / _trend;
		} else {
			factor = (1 - nextDouble) / (1 - _trend);
		}
		
		double change = factor * maximumChange;
		_mean = Math.min(Math.max(_mean + change, 0), _max);


        if (oldMean != _mean) {
            updatePastValuesMap();
        }

        Event.scheduleWithDelay(Time.SECOND, this, null, 0);
    }

    public void updatePastValuesMap() {
        _pastValues.put(Time.getCurrentTime(), _mean);
        _availableTimestamps = new ArrayList<>(_pastValues.keySet());
        Collections.sort(_availableTimestamps);
    }

	@Override
	public double getMaxValue() {
		return _max;
	}

}
